# coding: utf-8

# flake8: noqa

"""
    LIFF server API

    LIFF Server API.  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.liff.api.liff import Liff

from linebot.v3.liff.api.async_liff import AsyncLiff


# import ApiClient
from linebot.v3.liff.api_response import ApiResponse
from linebot.v3.liff.api_client import ApiClient
from linebot.v3.liff.async_api_client import AsyncApiClient
from linebot.v3.liff.configuration import Configuration
from linebot.v3.liff.exceptions import OpenApiException
from linebot.v3.liff.exceptions import ApiTypeError
from linebot.v3.liff.exceptions import ApiValueError
from linebot.v3.liff.exceptions import ApiKeyError
from linebot.v3.liff.exceptions import ApiAttributeError
from linebot.v3.liff.exceptions import ApiException

# import models into sdk package
from linebot.v3.liff.models.add_liff_app_request import AddLiffAppRequest
from linebot.v3.liff.models.add_liff_app_response import AddLiffAppResponse
from linebot.v3.liff.models.get_all_liff_apps_response import GetAllLiffAppsResponse
from linebot.v3.liff.models.liff_app import LiffApp
from linebot.v3.liff.models.liff_bot_prompt import LiffBotPrompt
from linebot.v3.liff.models.liff_features import LiffFeatures
from linebot.v3.liff.models.liff_scope import LiffScope
from linebot.v3.liff.models.liff_view import LiffView
from linebot.v3.liff.models.update_liff_app_request import UpdateLiffAppRequest
from linebot.v3.liff.models.update_liff_view import UpdateLiffView



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import BaseModel, Field, StrictInt, StrictStr, validator

class NumberOfMessagesResponse(BaseModel):
    """
    NumberOfMessagesResponse
    """
    status: StrictStr = Field(..., description="Aggregation process status. One of:  `ready`: The number of messages can be obtained. `unready`: We haven't finished calculating the number of sent messages for the specified in date. For example, this property is returned when the delivery date or a future date is specified. Calculation usually takes about a day. `unavailable_for_privacy`: The total number of messages on the specified day is less than 20. `out_of_service`: The specified date is earlier than the date on which we first started calculating sent messages (March 31, 2018). ")
    success: Optional[StrictInt] = Field(None, description="The number of messages delivered using the phone number on the date specified in `date`. The response has this property only when the value of `status` is `ready`.  ")

    __properties = ["status", "success"]

    @validator('status')
    def status_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('ready', 'unready', 'unavailable_for_privacy', 'out_of_service'):
            raise ValueError("must be one of enum values ('ready', 'unready', 'unavailable_for_privacy', 'out_of_service')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> NumberOfMessagesResponse:
        """Create an instance of NumberOfMessagesResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> NumberOfMessagesResponse:
        """Create an instance of NumberOfMessagesResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return NumberOfMessagesResponse.parse_obj(obj)

        _obj = NumberOfMessagesResponse.parse_obj({
            "status": obj.get("status"),
            "success": obj.get("success")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.limit module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class Recipient(with_metaclass(ABCMeta, Base)):
    """Recipient.

    https://developers.line.biz/en/reference/messaging-api/#narrowcast-recipient

    Recipient objects represent audiences. You can specify recipients based on
    a combination of criteria using logical operator objects.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Recipient, self).__init__(**kwargs)

        self.type = None


class AudienceRecipient(Recipient):
    """AudienceRecipient."""

    def __init__(self, group_id=None, **kwargs):
        """__init__ method.

        :param int group_id: The audience ID. Create audiences with the
            Manage Audience API.
        :param kwargs:
        """
        super(AudienceRecipient, self).__init__(**kwargs)

        self.type = "audience"
        self.audience_group_id = group_id


class RedeliveryRecipient(Recipient):
    """RedeliveryRecipient."""

    def __init__(self, request_id=None, **kwargs):
        """__init__ method.

        :param str request_id: The request ID of the narrowcast message previously sent.
            The request IDs is an ID issued for each Messaging API request.
        :param kwargs:
        """
        super(RedeliveryRecipient, self).__init__(**kwargs)

        self.type = "redelivery"
        self.request_id = request_id



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models package."""

from .actions import (  # noqa
    Action,
    PostbackAction,
    MessageAction,
    URIAction,
    DatetimePickerAction,
    CameraAction,
    CameraRollAction,
    LocationAction,
    RichMenuSwitchAction,
    Action as TemplateAction,  # backward compatibility
    PostbackAction as PostbackTemplateAction,  # backward compatibility
    MessageAction as MessageTemplateAction,  # backward compatibility
    URIAction as URITemplateAction,  # backward compatibility
    DatetimePickerAction as DatetimePickerTemplateAction,  # backward compatibility
    AltUri,
)
from .base import (  # noqa
    Base,
)
from .error import (  # noqa
    Error,
    ErrorDetail,
)
from .events import (  # noqa
    Event,
    MessageEvent,
    FollowEvent,
    UnfollowEvent,
    JoinEvent,
    LeaveEvent,
    PostbackEvent,
    AccountLinkEvent,
    MemberJoinedEvent,
    MemberLeftEvent,
    BeaconEvent,
    ThingsEvent,
    UnknownEvent,
    Postback,
    Beacon,
    Link,
    VideoPlayCompleteEvent,
    UnsendEvent,
)
from .filter import (  # noqa
    Filter,
    DemographicFilter,
    GenderFilter,
    AppTypeFilter,
    AreaFilter,
    AgeFilter,
    SubscriptionPeriodFilter,
)

from .flex_message import (  # noqa
    FlexSendMessage,
    FlexContainer,
    BubbleContainer,
    BubbleStyle,
    BlockStyle,
    CarouselContainer,
    FlexComponent,
    BoxComponent,
    ButtonComponent,
    FillerComponent,
    IconComponent,
    ImageComponent,
    SeparatorComponent,
    TextComponent,
    SpanComponent,
    VideoComponent
)
from .imagemap import (  # noqa
    ImagemapSendMessage,
    BaseSize,
    ImagemapAction,
    URIImagemapAction,
    MessageImagemapAction,
    ImagemapArea,
    Video,
    ExternalLink,
)
from .insight import (  # noqa
    DemographicInsight,
    AgeInsight,
    AreaInsight,
    AppTypeInsight,
    GenderInsight,
    SubscriptionPeriodInsight,
    MessageStatistics,
    MessageInsight,
    ClickInsight,
    JobInsight,
)

from .limit import (  # noqa
    Limit,
)

from .messages import (  # noqa
    Message,
    TextMessage,
    ImageMessage,
    VideoMessage,
    AudioMessage,
    LocationMessage,
    StickerMessage,
    FileMessage,
)

from .operator import (  # noqa
    And,
    Or,
    Not
)

from .recipient import (  # noqa
    AudienceRecipient,
    RedeliveryRecipient
)

from .responses import (  # noqa
    Profile,
    MemberIds,
    Content,
    RichMenuResponse,
    MessageQuotaResponse,
    MessageQuotaConsumptionResponse,
    MessageDeliveryBroadcastResponse,
    MessageDeliveryReplyResponse,
    MessageDeliveryMulticastResponse,
    MessageDeliveryPushResponse,
    Content as MessageContent,  # backward compatibility,
    IssueLinkTokenResponse,
    IssueChannelTokenResponse,
    InsightMessageDeliveryResponse,
    InsightFollowersResponse,
    InsightDemographicResponse,
    InsightMessageEventResponse,
    BroadcastResponse,
    NarrowcastResponse,
    ValidateReplyMessageObjectsResponse,
    ValidatePushMessageObjectsResponse,
    ValidateMulticastMessageObjectsResponse,
    ValidateBroadcastMessageObjectsResponse,
    ValidateNarrowcastMessageObjectsResponse,
    MessageProgressNarrowcastResponse,
    BotInfo,
    GetWebhookResponse,
    TestWebhookResponse,
    AudienceGroup,
    ClickAudienceGroup,
    ImpAudienceGroup,
    GetAuthorityLevel,
    Audience,
    CreateAudienceGroup,
)
from .rich_menu import (  # noqa
    RichMenu,
    RichMenuSize,
    RichMenuArea,
    RichMenuBounds,
    RichMenuAlias,
)
from .send_messages import (  # noqa
    SendMessage,
    TextSendMessage,
    ImageSendMessage,
    VideoSendMessage,
    AudioSendMessage,
    LocationSendMessage,
    StickerSendMessage,
    QuickReply,
    QuickReplyButton,
    Sender,
)
from .sources import (  # noqa
    Source,
    SourceUser,
    SourceGroup,
    SourceRoom,
)
from .template import (  # noqa
    TemplateSendMessage,
    Template,
    ButtonsTemplate,
    ConfirmTemplate,
    CarouselTemplate,
    CarouselColumn,
    ImageCarouselTemplate,
    ImageCarouselColumn,
)
from .things import (  # noqa
    DeviceLink,
    DeviceUnlink,
    ScenarioResult,
    ActionResult,
    Things,
)

from .background import (  # noqa
    Background,
    LinearGradientBackground,
)

from .emojis import (  # noqa
    Emojis
)



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.actions module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


def get_action(action):
    """Get action."""
    action_obj = Base.get_or_new_from_json_dict_with_types(
        action, {
            'postback': PostbackAction,
            'message': MessageAction,
            'uri': URIAction,
            'datetimepicker': DatetimePickerAction,
            'camera': CameraAction,
            'cameraRoll': CameraRollAction,
            'location': LocationAction,
            'richmenuswitch': RichMenuSwitchAction,
        }
    )
    return action_obj


def get_actions(actions):
    """Get actions."""
    new_actions = []
    if actions:
        for action in actions:
            action_obj = get_action(action)
            if action_obj:
                new_actions.append(action_obj)

    return new_actions


class Action(with_metaclass(ABCMeta, Base)):
    """Abstract base class of Action."""

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(Action, self).__init__(**kwargs)

        self.type = None


class PostbackAction(Action):
    """PostbackAction.

    https://developers.line.me/en/docs/messaging-api/reference/#postback-action

    When a control associated with this action is tapped,
    a postback event is returned via webhook with the specified string in the data property.
    """

    def __init__(
        self,
        label=None,
        data=None,
        display_text=None,
        text=None,
        input_option=None,
        fill_in_text=None,
        **kwargs
    ):
        """__init__ method.

        :param str label: Label for the action.
        :param str data: String returned via webhook
            in the postback.data property of the postback event.
        :param str display_text: Text displayed in the chat as a message sent by
            the user when the action is performed.
        :param str text: Deprecated. Text displayed in the chat as a message sent by
            the user when the action is performed. Returned from the server through a webhook.
        :param kwargs:
        """
        super(PostbackAction, self).__init__(**kwargs)

        self.type = 'postback'
        self.label = label
        self.data = data
        self.display_text = display_text
        self.text = text
        self.input_option = input_option
        self.fill_in_text = fill_in_text


class MessageAction(Action):
    """MessageAction.

    https://developers.line.me/en/docs/messaging-api/reference/#message-action

    When a control associated with this action is tapped,
    the string in the text property is sent as a message from the user.
    """

    def __init__(self, label=None, text=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action.
        :param str text: Text sent when the action is performed.
        :param kwargs:
        """
        super(MessageAction, self).__init__(**kwargs)

        self.type = 'message'
        self.label = label
        self.text = text


class URIAction(Action):
    """URIAction.

    https://developers.line.me/en/docs/messaging-api/reference/#uri-action

    When a control associated with this action is tapped,
    the URI specified in the uri property is opened.
    """

    def __init__(self, label=None, uri=None, alt_uri=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
            Max: 20 characters
        :param str uri: URI opened when the action is performed.
        :param alt_uri: URI opened when the desktop app.
        :type alt_uri: T <= :py:class:`linebot.models.actions.AltUri`
        :param kwargs:
        """
        super(URIAction, self).__init__(**kwargs)

        self.type = 'uri'
        self.label = label
        self.uri = uri
        self.alt_uri = self.get_or_new_from_json_dict(alt_uri, AltUri)


class AltUri(with_metaclass(ABCMeta, Base)):
    """AltUri.

    https://github.com/line/line-bot-sdk-python/issues/155

    URI opened when the desktop app.
    """

    def __init__(self, desktop=None, **kwargs):
        """__init__ method.

        :param str desktop: URI opened on LINE for macOS and Windows
            when the action is performed.
            If the altUri.desktop property is set,
            the uri property is ignored on LINE for macOS and Windows.
        :param kwargs:
        """
        super(AltUri, self).__init__(**kwargs)

        self.desktop = desktop


class DatetimePickerAction(Action):
    """DatetimePickerAction.

    https://developers.line.me/en/docs/messaging-api/reference/#datetime-picker-action

    When a control associated with this action is tapped,
    a postback event is returned via webhook with the date and time
    selected by the user from the date and time selection dialog.
    The datetime picker action does not support time zones.
    """

    def __init__(self, label=None, data=None, mode=None,
                 initial=None, max=None, min=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
        :param str data: String returned via webhook
            in the postback.data property of the postback event
        :param str mode: Action mode
            date: Pick date
            time: Pick time
            datetime: Pick date and time
        :param str initial: Initial value of date or time
        :param str max: Largest date or time value that can be selected.
            Must be greater than the min value.
        :param str min: Smallest date or time value that can be selected.
            Must be less than the max value.
        :param kwargs:
        """
        super(DatetimePickerAction, self).__init__(**kwargs)

        self.type = 'datetimepicker'
        self.label = label
        self.data = data
        self.mode = mode
        self.initial = initial
        self.max = max
        self.min = min


class CameraAction(Action):
    """CameraAction.

    https://developers.line.me/en/reference/messaging-api/#camera-action

    This action can be configured only with quick reply buttons.
    When a button associated with this action is tapped,
    the camera screen in the LINE app is opened.
    """

    def __init__(self, label=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
        :param kwargs:
        """
        super(CameraAction, self).__init__(**kwargs)

        self.type = 'camera'
        self.label = label


class CameraRollAction(Action):
    """CameraRollAction.

    https://developers.line.me/en/reference/messaging-api/#camera-roll-action

    This action can be configured only with quick reply buttons.
    When a button associated with this action is tapped,
    the camera roll screen in the LINE app is opened.
    """

    def __init__(self, label=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
        :param kwargs:
        """
        super(CameraRollAction, self).__init__(**kwargs)

        self.type = 'cameraRoll'
        self.label = label


class LocationAction(Action):
    """LocationRollAction.

    https://developers.line.me/en/reference/messaging-api/#location-action

    This action can be configured only with quick reply buttons.
    When a button associated with this action is tapped,
    the location screen in the LINE app is opened.
    """

    def __init__(self, label=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
        :param kwargs:
        """
        super(LocationAction, self).__init__(**kwargs)

        self.type = 'location'
        self.label = label


class RichMenuSwitchAction(Action):
    """RichMenuSwitchAction.

    https://developers.line.biz/en/reference/messaging-api/#richmenu-switch-action

    This action can be configured only with rich menus.
    It can't be used for Flex Messages or quick replies.
    When you tap a rich menu associated with this action,
    you can switch between rich menus,
    and a postback event including the rich menu alias ID selected
     by the user is returned via a webhook.
    """

    def __init__(self, label=None, rich_menu_alias_id=None, data=None, **kwargs):
        """__init__ method.

        :param str label: Label for the action
        :param str rich_menu_alias_id: Rich menu alias ID to switch to.
        :param str data: String returned by the postback.data property
         of the postback event via a webhook
        :param kwargs:
        """
        super(RichMenuSwitchAction, self).__init__(**kwargs)

        self.type = 'richmenuswitch'
        self.label = label
        self.rich_menu_alias_id = rich_menu_alias_id
        self.data = data



# coding: utf-8

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import re  # noqa: F401
import io

from pydantic.v1 import validate_arguments, ValidationError
from typing_extensions import Annotated

from pydantic.v1 import Field, constr, validator

from typing import Optional

from linebot.v3.insight.models.get_friends_demographics_response import GetFriendsDemographicsResponse
from linebot.v3.insight.models.get_message_event_response import GetMessageEventResponse
from linebot.v3.insight.models.get_number_of_followers_response import GetNumberOfFollowersResponse
from linebot.v3.insight.models.get_number_of_message_deliveries_response import GetNumberOfMessageDeliveriesResponse
from linebot.v3.insight.models.get_statistics_per_unit_response import GetStatisticsPerUnitResponse

from linebot.v3.insight.api_client import ApiClient
from linebot.v3.insight.api_response import ApiResponse
from linebot.v3.insight.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class Insight(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.line_base_path = "https://api.line.me"


    @validate_arguments
    def get_friends_demographics(self, **kwargs) -> GetFriendsDemographicsResponse:  # noqa: E501
        """get_friends_demographics  # noqa: E501

        Retrieves the demographic attributes for a LINE Official Account's friends.You can only retrieve information about friends for LINE Official Accounts created by users in Japan (JP), Thailand (TH), Taiwan (TW) and Indonesia (ID).   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_friends_demographics(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetFriendsDemographicsResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_friends_demographics_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_friends_demographics_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    def get_friends_demographics_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """get_friends_demographics  # noqa: E501

        Retrieves the demographic attributes for a LINE Official Account's friends.You can only retrieve information about friends for LINE Official Accounts created by users in Japan (JP), Thailand (TH), Taiwan (TW) and Indonesia (ID).   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_friends_demographics_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetFriendsDemographicsResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_friends_demographics" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetFriendsDemographicsResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/insight/demographic', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_message_event(self, request_id : Annotated[constr(strict=True, min_length=1), Field(..., description="Request ID of a narrowcast message or broadcast message. Each Messaging API request has a request ID. ")], **kwargs) -> GetMessageEventResponse:  # noqa: E501
        """Get user interaction statistics  # noqa: E501

        Returns statistics about how users interact with narrowcast messages or broadcast messages sent from your LINE Official Account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_event(request_id, async_req=True)
        >>> result = thread.get()

        :param request_id: Request ID of a narrowcast message or broadcast message. Each Messaging API request has a request ID.  (required)
        :type request_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetMessageEventResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_message_event_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_message_event_with_http_info(request_id, **kwargs)  # noqa: E501

    @validate_arguments
    def get_message_event_with_http_info(self, request_id : Annotated[constr(strict=True, min_length=1), Field(..., description="Request ID of a narrowcast message or broadcast message. Each Messaging API request has a request ID. ")], **kwargs) -> ApiResponse:  # noqa: E501
        """Get user interaction statistics  # noqa: E501

        Returns statistics about how users interact with narrowcast messages or broadcast messages sent from your LINE Official Account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_message_event_with_http_info(request_id, async_req=True)
        >>> result = thread.get()

        :param request_id: Request ID of a narrowcast message or broadcast message. Each Messaging API request has a request ID.  (required)
        :type request_id: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetMessageEventResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'request_id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_message_event" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('request_id') is not None:  # noqa: E501
            _query_params.append(('requestId', _params['request_id']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetMessageEventResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/insight/message/event', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_number_of_followers(self, var_date : Annotated[Optional[constr(strict=True, max_length=8, min_length=8)], Field(description="Date for which to retrieve the number of followers.  Format: yyyyMMdd (e.g. 20191231) Timezone: UTC+9 ")] = None, **kwargs) -> GetNumberOfFollowersResponse:  # noqa: E501
        """Get number of followers  # noqa: E501

        Returns the number of users who have added the LINE Official Account on or before a specified date.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_number_of_followers(var_date, async_req=True)
        >>> result = thread.get()

        :param var_date: Date for which to retrieve the number of followers.  Format: yyyyMMdd (e.g. 20191231) Timezone: UTC+9 
        :type var_date: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetNumberOfFollowersResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_number_of_followers_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_number_of_followers_with_http_info(var_date, **kwargs)  # noqa: E501

    @validate_arguments
    def get_number_of_followers_with_http_info(self, var_date : Annotated[Optional[constr(strict=True, max_length=8, min_length=8)], Field(description="Date for which to retrieve the number of followers.  Format: yyyyMMdd (e.g. 20191231) Timezone: UTC+9 ")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """Get number of followers  # noqa: E501

        Returns the number of users who have added the LINE Official Account on or before a specified date.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_number_of_followers_with_http_info(var_date, async_req=True)
        >>> result = thread.get()

        :param var_date: Date for which to retrieve the number of followers.  Format: yyyyMMdd (e.g. 20191231) Timezone: UTC+9 
        :type var_date: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetNumberOfFollowersResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'var_date'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_number_of_followers" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('var_date') is not None:  # noqa: E501
            _query_params.append(('date', _params['var_date']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetNumberOfFollowersResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/insight/followers', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_number_of_message_deliveries(self, var_date : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="Date for which to retrieve number of sent messages. - Format: yyyyMMdd (e.g. 20191231) - Timezone: UTC+9 ")], **kwargs) -> GetNumberOfMessageDeliveriesResponse:  # noqa: E501
        """Get number of message deliveries  # noqa: E501

        Returns the number of messages sent from LINE Official Account on a specified day.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_number_of_message_deliveries(var_date, async_req=True)
        >>> result = thread.get()

        :param var_date: Date for which to retrieve number of sent messages. - Format: yyyyMMdd (e.g. 20191231) - Timezone: UTC+9  (required)
        :type var_date: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetNumberOfMessageDeliveriesResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_number_of_message_deliveries_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_number_of_message_deliveries_with_http_info(var_date, **kwargs)  # noqa: E501

    @validate_arguments
    def get_number_of_message_deliveries_with_http_info(self, var_date : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="Date for which to retrieve number of sent messages. - Format: yyyyMMdd (e.g. 20191231) - Timezone: UTC+9 ")], **kwargs) -> ApiResponse:  # noqa: E501
        """Get number of message deliveries  # noqa: E501

        Returns the number of messages sent from LINE Official Account on a specified day.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_number_of_message_deliveries_with_http_info(var_date, async_req=True)
        >>> result = thread.get()

        :param var_date: Date for which to retrieve number of sent messages. - Format: yyyyMMdd (e.g. 20191231) - Timezone: UTC+9  (required)
        :type var_date: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetNumberOfMessageDeliveriesResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'var_date'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_number_of_message_deliveries" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('var_date') is not None:  # noqa: E501
            _query_params.append(('date', _params['var_date']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetNumberOfMessageDeliveriesResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/insight/message/delivery', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def get_statistics_per_unit(self, custom_aggregation_unit : Annotated[constr(strict=True, max_length=30, min_length=1), Field(..., description="Name of aggregation unit specified when sending the message. Case-sensitive. For example, `Promotion_a` and `Promotion_A` are regarded as different unit names. ")], var_from : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="Start date of aggregation period.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9 ")], to : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="End date of aggregation period. The end date can be specified for up to 30 days later. For example, if the start date is 20210301, the latest end date is 20210331.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9 ")], **kwargs) -> GetStatisticsPerUnitResponse:  # noqa: E501
        """get_statistics_per_unit  # noqa: E501

        You can check the per-unit statistics of how users interact with push messages and multicast messages sent from your LINE Official Account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_statistics_per_unit(custom_aggregation_unit, var_from, to, async_req=True)
        >>> result = thread.get()

        :param custom_aggregation_unit: Name of aggregation unit specified when sending the message. Case-sensitive. For example, `Promotion_a` and `Promotion_A` are regarded as different unit names.  (required)
        :type custom_aggregation_unit: str
        :param var_from: Start date of aggregation period.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9  (required)
        :type var_from: str
        :param to: End date of aggregation period. The end date can be specified for up to 30 days later. For example, if the start date is 20210301, the latest end date is 20210331.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9  (required)
        :type to: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: GetStatisticsPerUnitResponse
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            raise ValueError("Error! Please call the get_statistics_per_unit_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data")
        return self.get_statistics_per_unit_with_http_info(custom_aggregation_unit, var_from, to, **kwargs)  # noqa: E501

    @validate_arguments
    def get_statistics_per_unit_with_http_info(self, custom_aggregation_unit : Annotated[constr(strict=True, max_length=30, min_length=1), Field(..., description="Name of aggregation unit specified when sending the message. Case-sensitive. For example, `Promotion_a` and `Promotion_A` are regarded as different unit names. ")], var_from : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="Start date of aggregation period.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9 ")], to : Annotated[constr(strict=True, max_length=8, min_length=8), Field(..., description="End date of aggregation period. The end date can be specified for up to 30 days later. For example, if the start date is 20210301, the latest end date is 20210331.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9 ")], **kwargs) -> ApiResponse:  # noqa: E501
        """get_statistics_per_unit  # noqa: E501

        You can check the per-unit statistics of how users interact with push messages and multicast messages sent from your LINE Official Account.   # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.get_statistics_per_unit_with_http_info(custom_aggregation_unit, var_from, to, async_req=True)
        >>> result = thread.get()

        :param custom_aggregation_unit: Name of aggregation unit specified when sending the message. Case-sensitive. For example, `Promotion_a` and `Promotion_A` are regarded as different unit names.  (required)
        :type custom_aggregation_unit: str
        :param var_from: Start date of aggregation period.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9  (required)
        :type var_from: str
        :param to: End date of aggregation period. The end date can be specified for up to 30 days later. For example, if the start date is 20210301, the latest end date is 20210331.  Format: yyyyMMdd (e.g. 20210301) Time zone: UTC+9  (required)
        :type to: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(GetStatisticsPerUnitResponse, status_code(int), headers(HTTPHeaderDict))
        """

        _host = self.line_base_path
        _params = locals()

        _all_params = [
            'custom_aggregation_unit',
            'var_from',
            'to'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_statistics_per_unit" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('custom_aggregation_unit') is not None:  # noqa: E501
            _query_params.append(('customAggregationUnit', _params['custom_aggregation_unit']))

        if _params.get('var_from') is not None:  # noqa: E501
            _query_params.append(('from', _params['var_from']))

        if _params.get('to') is not None:  # noqa: E501
            _query_params.append(('to', _params['to']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['Bearer']  # noqa: E501

        _response_types_map = {
            '200': "GetStatisticsPerUnitResponse",
        }

        return self.api_client.call_api(
            '/v2/bot/insight/message/event/aggregation', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            _host=_host,
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.filter module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class Filter(with_metaclass(ABCMeta, Base)):
    """Filter.

    https://developers.line.biz/en/reference/messaging-api/#narrowcast-demographic-filter

    A filter is the top-level structure of a demographic element.
    """

    def __init__(self, demographic=None, **kwargs):
        """__init__ method.

        :param demographic: Combination of different criteria using logical
            operator objects.
        :type demographic: :py:class:`linebot.model.filter.DemographicFilter` |
            :py:class:`linebot.model.operator.Operator`
        :param kwargs:
        """
        super(Filter, self).__init__(**kwargs)

        self.demographic = demographic


class DemographicFilter(Filter):
    """DemographicFilter.

    https://developers.line.biz/en/reference/messaging-api/#narrowcast-demographic-filter

    Demographic filter objects represent criteria (e.g. age, gender, OS, region,
    and friendship duration) on which to filter the list of recipients.
    You can filter recipients based on a combination of different criteria using
    logical operator objects.
    """

    def __init__(self, **kwargs):
        """__init__ method.

        :param kwargs:
        """
        super(DemographicFilter, self).__init__(**kwargs)

        self.type = None


class GenderFilter(DemographicFilter):
    """GenderFilter."""

    def __init__(self, one_of=None, **kwargs):
        """__init__ method.

        :param one_of: Send messages to users of a given gender. One of:
            male: Users who identify as male
            female: Users who identify as female
        :type one_of: list[str]
        """
        super(GenderFilter, self).__init__(**kwargs)

        if one_of is None:
            one_of = []

        self.type = "gender"
        self.one_of = one_of


class AppTypeFilter(DemographicFilter):
    """AppTypeFilter."""

    def __init__(self, one_of=None, **kwargs):
        """__init__ method.

        :param one_of: Send messages to users of the specified OS. One of:
            ios: Users who using iOS.
            android: Users who using Android.
        :type one_of: list[str]
        """
        super(AppTypeFilter, self).__init__(**kwargs)

        if one_of is None:
            one_of = []

        self.type = "appType"
        self.one_of = one_of


class AreaFilter(DemographicFilter):
    """AreaFilter."""

    def __init__(self, one_of=None, **kwargs):
        """__init__ method.

        :param one_of: Send messages to users in the specified region.
        :type one_of: list[str]
        """
        super(AreaFilter, self).__init__(**kwargs)

        if one_of is None:
            one_of = []

        self.type = "area"
        self.one_of = one_of


class AgeFilter(DemographicFilter):
    """AgeFilter.

    This lets you filter recipients with a given age range.
    """

    def __init__(self, gte=None, lt=None, **kwargs):
        """__init__ method.

        Be sure to specify either gte, lt, or both.

        :param gte: Send messages to users at least as old as the specified age.
        :type gte: str
        :param lt: Send messages to users younger than the specified age.
            You can specify the same values as for the gte property.
        :type lt: str
        """
        super(AgeFilter, self).__init__(**kwargs)

        self.type = "age"
        self.gte = gte
        self.lt = lt


class SubscriptionPeriodFilter(DemographicFilter):
    """SubscriptionPeriodFilter.

    This lets you filter recipients with a given range of friendship durations.
    """

    def __init__(self, gte=None, lt=None, **kwargs):
        """__init__ method.

        Be sure to specify either gte, lt, or both.

        :param gte: Send messages to users who have been friends of yours for
            at least the specified number of days
        :type gte: str
        :param lt: Send messages to users who have been friends of yours for
            less than the specified number of days.
            You can specify the same values as for the gte property.
        :type lt: str
        """
        super(SubscriptionPeriodFilter, self).__init__(**kwargs)

        self.type = "subscriptionPeriod"
        self.gte = gte
        self.lt = lt



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictStr, conlist

class MembersIdsResponse(BaseModel):
    """
    MembersIdsResponse
    """
    member_ids: conlist(StrictStr, max_items=100) = Field(..., alias="memberIds", description="List of user IDs of members in the group chat. Only users of LINE for iOS and LINE for Android are included in `memberIds`.")
    next: Optional[StrictStr] = Field(None, description="A continuation token to get the next array of user IDs of the members in the group chat. Returned only when there are remaining user IDs that were not returned in `memberIds` in the original request. ")

    __properties = ["memberIds", "next"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> MembersIdsResponse:
        """Create an instance of MembersIdsResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> MembersIdsResponse:
        """Create an instance of MembersIdsResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return MembersIdsResponse.parse_obj(obj)

        _obj = MembersIdsResponse.parse_obj({
            "member_ids": obj.get("memberIds"),
            "next": obj.get("next")
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.v3.exceptions module."""


from abc import ABCMeta

from future.utils import with_metaclass


class BaseError(with_metaclass(ABCMeta, Exception)):
    """Base Exception class."""

    def __init__(self, message='-'):
        """__init__ method.

        :param str message: Human readable message
        """
        self.message = message

    def __repr__(self):
        """repr."""
        return str(self)

    def __str__(self):
        """str.

        :rtype: str
        """
        return '<{0} [{1}]>'.format(
            self.__class__.__name__, self.message)


class InvalidSignatureError(BaseError):
    """When Webhook signature does NOT match, this error will be raised."""

    def __init__(self, message='-'):
        """__init__ method.

        :param str message: Human readable message
        """
        super(InvalidSignatureError, self).__init__(message)



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest
from linebot.constants.postback_input_option import PostbackInputOption

from linebot.models import (
    PostbackAction,
    MessageAction,
    CameraRollAction,
    CameraAction,
    URIAction,
    LocationAction,
    DatetimePickerAction,
    AltUri,
)
from tests.models.serialize_test_case import SerializeTestCase


class TestActions(SerializeTestCase):
    def test_postback(self):
        arg = {
            'label': 'Buy',
            'data': 'action=buy&id=1',
            'display_text': 'buy',
            'input_option': PostbackInputOption.OPEN_KEYBOARD,
            'fill_in_text': 'fill in text',
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.POSTBACK),
            PostbackAction(**arg).as_json_dict()
        )

    def test_message(self):
        arg = {
            'label': 'Yes',
            'text': 'yes'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.MESSAGE),
            MessageAction(**arg).as_json_dict()
        )

    def test_camera(self):
        arg = {
            'label': 'camera'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.CAMERA),
            CameraAction(**arg).as_json_dict()
        )

    def test_camera_roll(self):
        arg = {
            'label': 'camera roll'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.CAMERA_ROLL),
            CameraRollAction(**arg).as_json_dict()
        )

    def test_datetime_picker(self):
        arg = {
            'label': 'Select date',
            'data': 'storeId=12345',
            'mode': 'datetime',
            'initial': '2017-12-25t00:00',
            'max': '2018-01-24t23:59',
            'min': '2017-12-25t00:00'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.DATETIMEPICKER),
            DatetimePickerAction(**arg).as_json_dict()
        )

    def test_uri(self):
        arg = {
            'label': 'View detail',
            'uri': 'https://example.com',
            'alt_uri': AltUri(desktop='https://example.com')
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.URI),
            URIAction(**arg).as_json_dict()
        )

    def test_location(self):
        arg = {
            'label': 'Location'
        }
        self.assertEqual(
            self.serialize_as_dict(arg, type=self.LOCATION),
            LocationAction(**arg).as_json_dict()
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    TextSendMessage
)


class TestSendTestMessage(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        # test data
        self.text_message = TextSendMessage(text='Hello, world')
        self.message = [{"type": "text", "text": "Hello, world"}]

    @responses.activate
    def test_validate_push_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/push',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.validate_push_message_objects(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/push')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def test_validate_reply_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/reply',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.validate_reply_message_objects(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/reply')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def test_validate_multicast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/multicast',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.validate_multicast_message_objects(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/multicast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def validate_test_broadcast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/broadcast',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.validate_broadcast_message_objects(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/broadcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def validate_test_narrowcast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/narrowcast',
            json={}, status=200, headers={'X-Line-Request-Id': 'request_id_test'}
        )

        response = self.tested.validate_narrowcast_message_objects(self.text_message)

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/validate/narrowcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message
            }
        )
        self.assertEqual('request_id_test', response.request_id)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import unittest

import responses

from linebot import (
    LineBotApi
)


class TestLineBotApi(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')

    @responses.activate
    def test_get_webhook_endpoint(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/channel/webhook/endpoint',
            json={
                "endpoint": "https://example.herokuapp.com/test",
                "active": True,
            },
            status=200
        )

        webhook = self.tested.get_webhook_endpoint()

        request = responses.calls[0].request
        self.assertEqual(request.method, 'GET')
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/channel/webhook/endpoint')
        self.assertEqual(webhook.endpoint, 'https://example.herokuapp.com/test')
        self.assertEqual(webhook.active, True)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import BaseModel, Field, StrictBool, StrictStr, conlist
from linebot.v3.messaging.models.message import Message

class ReplyMessageRequest(BaseModel):
    """
    ReplyMessageRequest
    https://developers.line.biz/en/reference/messaging-api/#send-reply-message
    """
    reply_token: StrictStr = Field(..., alias="replyToken", description="replyToken received via webhook.")
    messages: conlist(Message, max_items=5, min_items=1) = Field(..., description="List of messages.")
    notification_disabled: Optional[StrictBool] = Field(False, alias="notificationDisabled", description="`true`: The user doesn’t receive a push notification when a message is sent. `false`: The user receives a push notification when the message is sent (unless they have disabled push notifications in LINE and/or their device). The default value is false. ")

    __properties = ["replyToken", "messages", "notificationDisabled"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> ReplyMessageRequest:
        """Create an instance of ReplyMessageRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of each item in messages (list)
        _items = []
        if self.messages:
            for _item in self.messages:
                if _item:
                    _items.append(_item.to_dict())
            _dict['messages'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> ReplyMessageRequest:
        """Create an instance of ReplyMessageRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return ReplyMessageRequest.parse_obj(obj)

        _obj = ReplyMessageRequest.parse_obj({
            "reply_token": obj.get("replyToken"),
            "messages": [Message.from_dict(_item) for _item in obj.get("messages")] if obj.get("messages") is not None else None,
            "notification_disabled": obj.get("notificationDisabled") if obj.get("notificationDisabled") is not None else False
        })
        return _obj




# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""linebot.models.insight module."""


from abc import ABCMeta

from future.utils import with_metaclass

from .base import Base


class DemographicInsight(with_metaclass(ABCMeta, Base)):
    """Abstract Base Class of DemographicInsight."""

    def __init__(self, percentage=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param kwargs:
        """
        super(DemographicInsight, self).__init__(**kwargs)
        self.percentage = percentage


class GenderInsight(DemographicInsight):
    """GenderInsight."""

    def __init__(self, percentage=None, gender=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param str gender: Gender
        :param kwargs:
        """
        super(GenderInsight, self).__init__(percentage=percentage, **kwargs)

        self.gender = gender


class AgeInsight(DemographicInsight):
    """AgeInsight."""

    def __init__(self, percentage=None, age=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param str age: Age
        :param kwargs:
        """
        super(AgeInsight, self).__init__(percentage=percentage, **kwargs)

        self.age = age


class AreaInsight(DemographicInsight):
    """AreaInsight."""

    def __init__(self, percentage=None, area=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param str area: Area
        :param kwargs:
        """
        super(AreaInsight, self).__init__(percentage=percentage, **kwargs)

        self.area = area


class AppTypeInsight(DemographicInsight):
    """AppTypeInsight."""

    def __init__(self, percentage=None, app_type=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param str app_type: OS
        :param kwargs:
        """
        super(AppTypeInsight, self).__init__(percentage=percentage, **kwargs)

        self.app_type = app_type


class SubscriptionPeriodInsight(DemographicInsight):
    """SubscriptionPeriodInsight."""

    def __init__(self, percentage=None, subscription_period=None, **kwargs):
        """__init__ method.

        :param float percentage: Percentage.
        :param str subscription_period: Friendship duration
        :param kwargs:
        """
        super(SubscriptionPeriodInsight, self).__init__(percentage=percentage, **kwargs)

        self.subscription_period = subscription_period


class MessageStatistics(Base):
    """MessageStatistics."""

    def __init__(self, request_id=None, timestamp=None, delivered=None,
                 unique_impression=None, unique_click=None, unique_media_played=None,
                 unique_media_played_100_percent=None, **kwargs):
        """__init__ method.

        :param str request_id: Request ID.
        :param int timestamp: UNIX timestamp for message delivery time.
        :param int delivered: Number of messages delivered. This property shows values
            of less than 20.
        :param int unique_impression: Number of people who opened the message,
            meaning they displayed at least 1 bubble.
        :param int unique_click: Number of people who opened any URL in the message.
        :param int unique_media_played: Number of people who started playing any video
            or audio in the message.
        :param int unique_media_played_100_percent: Number of people who played the entirety of
            any video or audio in the message.
        """
        super(MessageStatistics, self).__init__(**kwargs)

        self.request_id = request_id
        self.timestamp = timestamp
        self.delivered = delivered
        self.unique_impression = unique_impression
        self.unique_click = unique_click
        self.unique_media_played = unique_media_played
        self.unique_media_played_100_percent = unique_media_played_100_percent


class MessageStatisticsOfCustomAggregationUnit(Base):
    """MessageStatisticsOfCustomAggregationUnit."""

    def __init__(self, unique_impression=None, unique_click=None, unique_media_played=None,
                 unique_media_played_100_percent=None, **kwargs):
        """__init__ method.

        :param int unique_impression: Number of people who opened the message,
            meaning they displayed at least 1 bubble.
        :param int unique_click: Number of people who opened any URL in the message.
        :param int unique_media_played: Number of people who started playing any video
            or audio in the message.
        :param int unique_media_played_100_percent: Number of people who played the entirety of
            any video or audio in the message.
        """
        super(MessageStatisticsOfCustomAggregationUnit, self).__init__(**kwargs)

        self.unique_impression = unique_impression
        self.unique_click = unique_click
        self.unique_media_played = unique_media_played
        self.unique_media_played_100_percent = unique_media_played_100_percent


class MessageInsight(Base):
    """MessageInsight."""

    def __init__(self, seq=None, impression=None, media_played=None,
                 media_played_25_percent=None, media_played_50_percent=None,
                 media_played_75_percent=None, media_played_100_percent=None,
                 unique_media_played=None, unique_media_played_25_percent=None,
                 unique_media_played_50_percent=None, unique_media_played_75_percent=None,
                 unique_media_played_100_percent=None, **kwargs):
        """__init__ method.

        :param int seq: Bubble's serial number.
        :param int impression: Number of times the bubble was displayed.
        :param int media_played: Number of times audio or video in the bubble started playing.
        :param int media_played_25_percent: Number of times audio or video
            in the bubble was played from start to 25%.
        :param int media_played_50_percent: Number of times audio or video
            in the bubble was played from start to 50%.
        :param int media_played_75_percent: Number of times audio or video
            in the bubble was played from start to 75%.
        :param int media_played_100_percent: Number of times audio or video
            in the bubble was played in its entirety.
        :param int unique_media_played: Number of people that started playing
            audio or video in the bubble.
        :param int unique_media_played_25_percent: Number of people that played
            audio or video in the bubble from start to 25%.
        :param int unique_media_played_50_percent: Number of people that played
            audio or video in the bubble from start to 50%.
        :param int unique_media_played_75_percent: Number of people that played
            audio or video in the bubble from start to 75%.
        :param int unique_media_played_100_percent: Number of people that played
            audio or video in the bubble in its entirety.
        """
        super(MessageInsight, self).__init__(**kwargs)

        self.seq = seq
        self.impression = impression
        self.media_played = media_played
        self.media_played_25_percent = media_played_25_percent
        self.media_played_50_percent = media_played_50_percent
        self.media_played_75_percent = media_played_75_percent
        self.media_played_100_percent = media_played_100_percent
        self.unique_media_played = unique_media_played
        self.unique_media_played_25_percent = unique_media_played_25_percent
        self.unique_media_played_50_percent = unique_media_played_50_percent
        self.unique_media_played_75_percent = unique_media_played_75_percent
        self.unique_media_played_100_percent = unique_media_played_100_percent


class ClickInsight(Base):
    """ClickInsight."""

    def __init__(self, seq=None, url=None, click=None, unique_click=None,
                 unique_click_of_request=None, **kwargs):
        """__init__ method.

        :param int seq: The URL's serial number.
        :param str url: URL.
        :param int click: Number of times the URL was opened.
        :param int unique_click: Number of people that opened the URL.
        :param int unique_click_of_request: Number of people who opened this url
            through any link in the message.
        """
        super(ClickInsight, self).__init__(**kwargs)

        self.seq = seq
        self.url = url
        self.click = click
        self.unique_click = unique_click
        self.unique_click_of_request = unique_click_of_request


class JobInsight(Base):
    """ClickInsight."""

    def __init__(self, audience_group_job_id=None, audience_group_id=None, description=None,
                 type=None, job_status=None, failed_type=None, audience_count=None,
                 created=None, **kwargs):
        """__init__ method.

        :param int audience_group_job_id: A job ID.
        :param int audience_group_id: An audience ID.
        :param str description: The job's description.
        :param str type: The job's type. One of: 'DIFF_ADD'
        :param str job_status: The job's status. One of: 'QUEUED', 'WORKING', 'FINISHED', 'FAILED'
        :param str failed_type: The reason why the operation failed. This is only included when
            jobs[].jobStatus is FAILED.
        :param int audience_count: The number of accounts (recipients) that were added or removed.
        :param int created: When the job was created (in UNIX time).
        :param kwargs:
        """
        super(JobInsight, self).__init__(**kwargs)
        self.audience_group_job_id = audience_group_job_id
        self.audience_group_id = audience_group_id
        self.description = description
        self.type = type
        self.job_status = job_status
        self.failed_type = failed_type
        self.audience_count = audience_count
        self.created = created



# coding: utf-8

# flake8: noqa

"""
    LINE Messaging API(Insight)

    This document describes LINE Messaging API(Insight).  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.insight.api.insight import Insight

from linebot.v3.insight.api.async_insight import AsyncInsight


# import ApiClient
from linebot.v3.insight.api_response import ApiResponse
from linebot.v3.insight.api_client import ApiClient
from linebot.v3.insight.async_api_client import AsyncApiClient
from linebot.v3.insight.configuration import Configuration
from linebot.v3.insight.exceptions import OpenApiException
from linebot.v3.insight.exceptions import ApiTypeError
from linebot.v3.insight.exceptions import ApiValueError
from linebot.v3.insight.exceptions import ApiKeyError
from linebot.v3.insight.exceptions import ApiAttributeError
from linebot.v3.insight.exceptions import ApiException

# import models into sdk package
from linebot.v3.insight.models.age_tile import AgeTile
from linebot.v3.insight.models.app_type_tile import AppTypeTile
from linebot.v3.insight.models.area_tile import AreaTile
from linebot.v3.insight.models.error_detail import ErrorDetail
from linebot.v3.insight.models.error_response import ErrorResponse
from linebot.v3.insight.models.gender_tile import GenderTile
from linebot.v3.insight.models.get_friends_demographics_response import GetFriendsDemographicsResponse
from linebot.v3.insight.models.get_message_event_response import GetMessageEventResponse
from linebot.v3.insight.models.get_message_event_response_click import GetMessageEventResponseClick
from linebot.v3.insight.models.get_message_event_response_message import GetMessageEventResponseMessage
from linebot.v3.insight.models.get_message_event_response_overview import GetMessageEventResponseOverview
from linebot.v3.insight.models.get_number_of_followers_response import GetNumberOfFollowersResponse
from linebot.v3.insight.models.get_number_of_message_deliveries_response import GetNumberOfMessageDeliveriesResponse
from linebot.v3.insight.models.get_statistics_per_unit_response import GetStatisticsPerUnitResponse
from linebot.v3.insight.models.get_statistics_per_unit_response_click import GetStatisticsPerUnitResponseClick
from linebot.v3.insight.models.get_statistics_per_unit_response_message import GetStatisticsPerUnitResponseMessage
from linebot.v3.insight.models.get_statistics_per_unit_response_overview import GetStatisticsPerUnitResponseOverview
from linebot.v3.insight.models.subscription_period_tile import SubscriptionPeriodTile



# -*- coding: utf-8 -*-

#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       https://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

from __future__ import unicode_literals, absolute_import

import json
import unittest

import responses

from linebot import (
    LineBotApi
)
from linebot.models import (
    TextSendMessage,
    Limit,
    And,
    Or,
    Not,
    Filter,
    GenderFilter,
    AppTypeFilter,
    AreaFilter,
    AgeFilter,
    AudienceRecipient,
    SubscriptionPeriodFilter,
    RedeliveryRecipient,
)


class TestNarrowcastMessage(unittest.TestCase):
    def setUp(self):
        self.tested = LineBotApi('channel_secret')
        self.maxDiff = None
        self.request_id = 'f70dd685-499a-4231-a441-f24b8d4fba21'

        # test data
        self.text_message = TextSendMessage(text='Hello, world')
        self.message = [{"type": "text", "text": "Hello, world"}]

    @responses.activate
    def test_narrowcast_simple_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast',
            json={}, status=200,
            headers={'X-Line-Request-Id': 'request_id_test'},
        )

        self.tested.narrowcast(
            self.text_message,
            recipient=AudienceRecipient(group_id=5614991017776),
            filter=Filter(demographic=AgeFilter(gte="age_35", lt="age_40")),
            limit=Limit(max=10),
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message,
                "recipient": {
                    'audienceGroupId': 5614991017776,
                    'type': 'audience'
                },
                "filter": {
                    "demographic": {
                        "type": "age",
                        "gte": "age_35",
                        "lt": "age_40"
                    }
                },
                "limit": {
                    "max": 10,
                    "upToRemainingQuota": False,
                },
                "notificationDisabled": False,
            }
        )

    @responses.activate
    def test_narrowcast_redelivery_recipient_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast',
            json={}, status=200,
        )

        self.tested.narrowcast(
            self.text_message,
            recipient=And(
                AudienceRecipient(group_id=5614991017776),
                Not(
                    RedeliveryRecipient(request_id='request_id_test')
                )
            ),
            filter=Filter(demographic=AgeFilter(gte="age_35", lt="age_40")),
            limit=Limit(max=10),
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message,
                "recipient": {
                    "type": "operator",
                    "and": [
                        {
                            "type": "audience",
                            "audienceGroupId": 5614991017776
                        },
                        {
                            "type": "operator",
                            "not": {
                                "type": "redelivery",
                                "requestId": "request_id_test"
                            }
                        }
                    ]
                },
                "filter": {
                    "demographic": {
                        "type": "age",
                        "gte": "age_35",
                        "lt": "age_40"
                    }
                },
                "limit": {
                    "max": 10,
                    "upToRemainingQuota": False,
                },
                "notificationDisabled": False,
            }
        )

    @responses.activate
    def test_narrowcast_text_message(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast',
            json={}, status=200,
            headers={'X-Line-Request-Id': 'request_id_test'},
        )

        response = self.tested.narrowcast(
            self.text_message,
            recipient=And(
                AudienceRecipient(group_id=5614991017776),
                Not(AudienceRecipient(group_id=4389303728991))
            ),
            filter=Filter(
                demographic=Or(
                    And(
                        GenderFilter(one_of=["male", "female"]),
                        AgeFilter(gte="age_20", lt="age_25"),
                        AppTypeFilter(one_of=["android", "ios"]),
                        AreaFilter(one_of=["jp_23", "jp_05"]),
                        SubscriptionPeriodFilter(gte="day_7", lt="day_30")
                    ),
                    And(
                        AgeFilter(gte="age_35", lt="age_40"),
                        Not(GenderFilter(one_of=["male"]))
                    )
                )
            ),
            limit=Limit(max=100, up_to_remaining_quota=True),
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message,
                "recipient": {
                    "type": "operator",
                    "and": [
                        {
                            'audienceGroupId': 5614991017776,
                            'type': 'audience'
                        },
                        {
                            "type": "operator",
                            "not": {
                                "type": "audience",
                                "audienceGroupId": 4389303728991
                            }
                        }
                    ]
                },
                "filter": {
                    "demographic": {
                        "type": "operator",
                        "or": [
                            {
                                "type": "operator",
                                "and": [
                                    {
                                        "type": "gender",
                                        "oneOf": [
                                            "male",
                                            "female"
                                        ]
                                    },
                                    {
                                        "type": "age",
                                        "gte": "age_20",
                                        "lt": "age_25"
                                    },
                                    {
                                        "type": "appType",
                                        "oneOf": [
                                            "android",
                                            "ios"
                                        ]
                                    },
                                    {
                                        "type": "area",
                                        "oneOf": [
                                            "jp_23",
                                            "jp_05"
                                        ]
                                    },
                                    {
                                        "type": "subscriptionPeriod",
                                        "gte": "day_7",
                                        "lt": "day_30"
                                    }
                                ]
                            },
                            {
                                "type": "operator",
                                "and": [
                                    {
                                        "type": "age",
                                        "gte": "age_35",
                                        "lt": "age_40"
                                    },
                                    {
                                        "type": "operator",
                                        "not": {
                                            "type": "gender",
                                            "oneOf": [
                                                "male"
                                            ]
                                        }
                                    }
                                ]
                            }
                        ]
                    }
                },
                "limit": {
                    "max": 100,
                    "upToRemainingQuota": True,
                },
                "notificationDisabled": False,
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def test_narrowcast_text_message_with_retry_key(self):
        responses.add(
            responses.POST,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast',
            json={}, status=200,
            headers={
                'X-Line-Request-Id': 'request_id_test',
                'X-Line-Retry-Key': '123e4567-e89b-12d3-a456-426614174000'
            },
        )

        response = self.tested.narrowcast(
            self.text_message,
            recipient=And(
                AudienceRecipient(group_id=5614991017776),
                Not(AudienceRecipient(group_id=4389303728991))
            ),
            filter=Filter(
                demographic=Or(
                    And(
                        GenderFilter(one_of=["male", "female"])
                    )
                )
            ),
            limit=Limit(max=100, up_to_remaining_quota=True),
            retry_key='123e4567-e89b-12d3-a456-426614174000',
        )

        request = responses.calls[0].request
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT + '/v2/bot/message/narrowcast')
        self.assertEqual(request.method, 'POST')
        self.assertEqual(
            request.headers['X-Line-Retry-Key'],
            '123e4567-e89b-12d3-a456-426614174000'
        )
        self.assertEqual(
            json.loads(request.body),
            {
                "messages": self.message,
                "recipient": {
                    "type": "operator",
                    "and": [
                        {
                            'audienceGroupId': 5614991017776,
                            'type': 'audience'
                        },
                        {
                            "type": "operator",
                            "not": {
                                "type": "audience",
                                "audienceGroupId": 4389303728991
                            }
                        }
                    ]
                },
                "filter": {
                    "demographic": {
                        "type": "operator",
                        "or": [
                            {
                                "type": "operator",
                                "and": [
                                    {
                                        "type": "gender",
                                        "oneOf": [
                                            "male",
                                            "female"
                                        ]
                                    }
                                ]
                            }
                        ]
                    }
                },
                "limit": {
                    "max": 100,
                    "upToRemainingQuota": True,
                },
                "notificationDisabled": False,
            }
        )
        self.assertEqual('request_id_test', response.request_id)

    @responses.activate
    def test_get_progress_status_narrowcast(self):
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/progress/narrowcast?requestId={request_id}'.format(
                request_id=self.request_id),
            json={'phase': 'waiting'}, status=200,
        )
        responses.add(
            responses.GET,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/progress/narrowcast?requestId={request_id}'.format(
                request_id=self.request_id),
            json={
                'phase': 'succeeded',
                'successCount': 10,
                'failureCount': 0,
                'targetCount': 10,
            }, status=200,
        )

        res = self.tested.get_progress_status_narrowcast(self.request_id)
        request = responses.calls[0].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/progress/narrowcast?requestId={request_id}'.format(
                request_id=self.request_id)
        )

        self.assertEqual(res.phase, 'waiting')

        res = self.tested.get_progress_status_narrowcast(self.request_id)
        request = responses.calls[1].request
        self.assertEqual('GET', request.method)
        self.assertEqual(
            request.url,
            LineBotApi.DEFAULT_API_ENDPOINT +
            '/v2/bot/message/progress/narrowcast?requestId={request_id}'.format(
                request_id=self.request_id)
        )

        self.assertEqual(res.phase, 'succeeded')
        self.assertEqual(res.success_count, 10)
        self.assertEqual(res.failure_count, 0)
        self.assertEqual(res.target_count, 10)


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

# flake8: noqa

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


# import apis into sdk package
from linebot.v3.webhooks.api.dummy import Dummy

from linebot.v3.webhooks.api.async_dummy import AsyncDummy


# import ApiClient
from linebot.v3.webhooks.api_response import ApiResponse
from linebot.v3.webhooks.api_client import ApiClient
from linebot.v3.webhooks.async_api_client import AsyncApiClient
from linebot.v3.webhooks.configuration import Configuration
from linebot.v3.webhooks.exceptions import OpenApiException
from linebot.v3.webhooks.exceptions import ApiTypeError
from linebot.v3.webhooks.exceptions import ApiValueError
from linebot.v3.webhooks.exceptions import ApiKeyError
from linebot.v3.webhooks.exceptions import ApiAttributeError
from linebot.v3.webhooks.exceptions import ApiException

# import models into sdk package
from linebot.v3.webhooks.models.account_link_event import AccountLinkEvent
from linebot.v3.webhooks.models.activated_event import ActivatedEvent
from linebot.v3.webhooks.models.all_mentionee import AllMentionee
from linebot.v3.webhooks.models.attached_module_content import AttachedModuleContent
from linebot.v3.webhooks.models.audio_message_content import AudioMessageContent
from linebot.v3.webhooks.models.beacon_content import BeaconContent
from linebot.v3.webhooks.models.beacon_event import BeaconEvent
from linebot.v3.webhooks.models.bot_resumed_event import BotResumedEvent
from linebot.v3.webhooks.models.bot_suspended_event import BotSuspendedEvent
from linebot.v3.webhooks.models.callback_request import CallbackRequest
from linebot.v3.webhooks.models.chat_control import ChatControl
from linebot.v3.webhooks.models.content_provider import ContentProvider
from linebot.v3.webhooks.models.deactivated_event import DeactivatedEvent
from linebot.v3.webhooks.models.delivery_context import DeliveryContext
from linebot.v3.webhooks.models.detached_module_content import DetachedModuleContent
from linebot.v3.webhooks.models.emoji import Emoji
from linebot.v3.webhooks.models.event import Event
from linebot.v3.webhooks.models.event_mode import EventMode
from linebot.v3.webhooks.models.file_message_content import FileMessageContent
from linebot.v3.webhooks.models.follow_detail import FollowDetail
from linebot.v3.webhooks.models.follow_event import FollowEvent
from linebot.v3.webhooks.models.group_source import GroupSource
from linebot.v3.webhooks.models.image_message_content import ImageMessageContent
from linebot.v3.webhooks.models.image_set import ImageSet
from linebot.v3.webhooks.models.join_event import JoinEvent
from linebot.v3.webhooks.models.joined_members import JoinedMembers
from linebot.v3.webhooks.models.joined_membership_content import JoinedMembershipContent
from linebot.v3.webhooks.models.leave_event import LeaveEvent
from linebot.v3.webhooks.models.left_members import LeftMembers
from linebot.v3.webhooks.models.left_membership_content import LeftMembershipContent
from linebot.v3.webhooks.models.link_content import LinkContent
from linebot.v3.webhooks.models.location_message_content import LocationMessageContent
from linebot.v3.webhooks.models.member_joined_event import MemberJoinedEvent
from linebot.v3.webhooks.models.member_left_event import MemberLeftEvent
from linebot.v3.webhooks.models.membership_content import MembershipContent
from linebot.v3.webhooks.models.membership_event import MembershipEvent
from linebot.v3.webhooks.models.mention import Mention
from linebot.v3.webhooks.models.mentionee import Mentionee
from linebot.v3.webhooks.models.message_content import MessageContent
from linebot.v3.webhooks.models.message_event import MessageEvent
from linebot.v3.webhooks.models.module_content import ModuleContent
from linebot.v3.webhooks.models.module_event import ModuleEvent
from linebot.v3.webhooks.models.pnp_delivery import PnpDelivery
from linebot.v3.webhooks.models.pnp_delivery_completion_event import PnpDeliveryCompletionEvent
from linebot.v3.webhooks.models.postback_content import PostbackContent
from linebot.v3.webhooks.models.postback_event import PostbackEvent
from linebot.v3.webhooks.models.renewed_membership_content import RenewedMembershipContent
from linebot.v3.webhooks.models.room_source import RoomSource
from linebot.v3.webhooks.models.source import Source
from linebot.v3.webhooks.models.sticker_message_content import StickerMessageContent
from linebot.v3.webhooks.models.text_message_content import TextMessageContent
from linebot.v3.webhooks.models.unfollow_event import UnfollowEvent
from linebot.v3.webhooks.models.unsend_detail import UnsendDetail
from linebot.v3.webhooks.models.unsend_event import UnsendEvent
from linebot.v3.webhooks.models.user_mentionee import UserMentionee
from linebot.v3.webhooks.models.user_source import UserSource
from linebot.v3.webhooks.models.video_message_content import VideoMessageContent
from linebot.v3.webhooks.models.video_play_complete import VideoPlayComplete
from linebot.v3.webhooks.models.video_play_complete_event import VideoPlayCompleteEvent



# flake8: noqa

# import apis into api package
from linebot.v3.audience.api.manage_audience import ManageAudience
from linebot.v3.audience.api.manage_audience_blob import ManageAudienceBlob


# Async version
from linebot.v3.audience.api.async_manage_audience import AsyncManageAudience
from linebot.v3.audience.api.async_manage_audience_blob import AsyncManageAudienceBlob




# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Optional
from pydantic.v1 import Field, StrictInt, StrictStr
from linebot.v3.webhooks.models.content_provider import ContentProvider
from linebot.v3.webhooks.models.message_content import MessageContent

class VideoMessageContent(MessageContent):
    """
    VideoMessageContent
    """
    duration: Optional[StrictInt] = Field(None, description="Length of video file (milliseconds)")
    content_provider: ContentProvider = Field(..., alias="contentProvider")
    quote_token: StrictStr = Field(..., alias="quoteToken", description="Quote token to quote this message. ")
    type: str = "video"

    __properties = ["type", "id", "duration", "contentProvider", "quoteToken"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> VideoMessageContent:
        """Create an instance of VideoMessageContent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic.v1 by calling `to_dict()` of content_provider
        if self.content_provider:
            _dict['contentProvider'] = self.content_provider.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> VideoMessageContent:
        """Create an instance of VideoMessageContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return VideoMessageContent.parse_obj(obj)

        _obj = VideoMessageContent.parse_obj({
            "type": obj.get("type"),
            "id": obj.get("id"),
            "duration": obj.get("duration"),
            "content_provider": ContentProvider.from_dict(obj.get("contentProvider")) if obj.get("contentProvider") is not None else None,
            "quote_token": obj.get("quoteToken")
        })
        return _obj




# coding: utf-8

"""
    LINE Messaging API

    This document describes LINE Messaging API.  # noqa: E501

    The version of the OpenAPI document: 0.0.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class FlexSpanSize(str, Enum):
    """
    Font size in the `size` property of the Flex span component. You can specify the size in pixels or with a keyword. FlexSpanSize just provides only keywords. 
    """

    """
    allowed enum values
    """
    XXS = 'xxs'
    XS = 'xs'
    SM = 'sm'
    MD = 'md'
    LG = 'lg'
    XL = 'xl'
    XXL = 'xxl'
    ENUM_3XL = '3xl'
    ENUM_4XL = '4xl'
    ENUM_5XL = '5xl'

    @classmethod
    def from_json(cls, json_str: str) -> FlexSpanSize:
        """Create an instance of FlexSpanSize from a JSON string"""
        return FlexSpanSize(json.loads(json_str))





# coding: utf-8

"""
    Webhook Type Definition

    Webhook event definition of the LINE Messaging API  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic.v1 import Field, StrictStr, conlist, constr, validator
from linebot.v3.webhooks.models.message_content import MessageContent

class StickerMessageContent(MessageContent):
    """
    StickerMessageContent
    https://developers.line.biz/en/reference/messaging-api/#wh-sticker
    """
    package_id: StrictStr = Field(..., alias="packageId", description="Package ID")
    sticker_id: StrictStr = Field(..., alias="stickerId", description="Sticker ID")
    sticker_resource_type: StrictStr = Field(..., alias="stickerResourceType")
    keywords: Optional[conlist(StrictStr, max_items=15)] = Field(None, description="Array of up to 15 keywords describing the sticker. If a sticker has 16 or more keywords, a random selection of 15 keywords will be returned. The keyword selection is random for each event, so different keywords may be returned for the same sticker. ")
    text: Optional[constr(strict=True, max_length=100)] = Field(None, description="Any text entered by the user. This property is only included for message stickers. Max character limit: 100 ")
    quote_token: StrictStr = Field(..., alias="quoteToken", description="Quote token to quote this message. ")
    quoted_message_id: Optional[StrictStr] = Field(None, alias="quotedMessageId", description="Message ID of a quoted message. Only included when the received message quotes a past message.  ")
    type: str = "sticker"

    __properties = ["type", "id", "packageId", "stickerId", "stickerResourceType", "keywords", "text", "quoteToken", "quotedMessageId"]

    @validator('sticker_resource_type')
    def sticker_resource_type_validate_enum(cls, value):
        """Validates the enum"""
        if value not in ('STATIC', 'ANIMATION', 'SOUND', 'ANIMATION_SOUND', 'POPUP', 'POPUP_SOUND', 'CUSTOM', 'MESSAGE', 'NAME_TEXT', 'PER_STICKER_TEXT'):
            raise ValueError("must be one of enum values ('STATIC', 'ANIMATION', 'SOUND', 'ANIMATION_SOUND', 'POPUP', 'POPUP_SOUND', 'CUSTOM', 'MESSAGE', 'NAME_TEXT', 'PER_STICKER_TEXT')")
        return value

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> StickerMessageContent:
        """Create an instance of StickerMessageContent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> StickerMessageContent:
        """Create an instance of StickerMessageContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return StickerMessageContent.parse_obj(obj)

        _obj = StickerMessageContent.parse_obj({
            "type": obj.get("type"),
            "id": obj.get("id"),
            "package_id": obj.get("packageId"),
            "sticker_id": obj.get("stickerId"),
            "sticker_resource_type": obj.get("stickerResourceType"),
            "keywords": obj.get("keywords"),
            "text": obj.get("text"),
            "quote_token": obj.get("quoteToken"),
            "quoted_message_id": obj.get("quotedMessageId")
        })
        return _obj

